<div class="controls">
<?= $this->tabs ?>
<h1><?= $this->escape($this->title) ?></h1>
</div>

<div class="content">
<p class="legacy-error"><?= $this->escape($this->message) ?></p>
<?= $this->table ?>
</div>
